/*
 
 FKTokenFieldCell.m
 FieldKit
 
 Copyright (cc) 2012 Luis Laugga.
 Some rights reserved, all wrongs deserved.
 
 Licensed under a Creative Commons Attribution-ShareAlike 3.0 License;
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://creativecommons.org/licenses/by-sa/3.0/
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 
*/

#import "FKTokenFieldCell.h"

@implementation FKTokenFieldCell

@synthesize font = _font;
@synthesize text = _text;

@synthesize representedObject = _representedObject;

@synthesize scaled = _scaled;
@synthesize size = _size;

#define kTokenFieldCellInsetLeft 8
#define kTokenFieldCellInsetTop 3
#define kTokenFieldCellStretchCap 12
#define kTokenFieldCellOpacity 1.0

#define kTokenFieldCellScaledFactor 1.2
#define kTokenFieldCellScaledInsetLeft 10
#define kTokenFieldCellScaledInsetTop 4
#define kTokenFieldCellScaledStretchCap 15
#define kTokenFieldCellScaledOpacity 0.9

#define kTokenFieldCellScaledAnimationDuration 0.15

#pragma mark -
#pragma mark Initialization

- (id)initWithText:(NSString *)text andFont:(UIFont *)font;
{ 
    self = [super init];
    if (self) 
    {
        CGSize textSize = [text sizeWithFont:font];
        _size = CGSizeMake(kTokenFieldCellInsetLeft + textSize.width + kTokenFieldCellInsetLeft, kTokenFieldCellInsetTop + textSize.height + kTokenFieldCellInsetTop);
        
        self.frame = CGRectMake(0, 0, _size.width, _size.height);
        self.selected = NO;
        
        self.text = text; // copy
        self.font = font; // assign
        
        self.backgroundColor = [UIColor clearColor];
        self.contentMode = UIViewContentModeRedraw; // drawRect: method invoked when the frame rectangle changes (ie. scaled = YES)
        
        self.representedObject = nil;
    }
    return self;
}

- (void)dealloc
{
    self.text = nil;
    self.font = nil;
    
    self.representedObject = nil; // release
    
    [super dealloc];
}

#pragma mark -
#pragma mark Properties

- (void)setSelected:(BOOL)selected
{
    [super setSelected:selected];
    
    // Update drawing
    [self setNeedsDisplay];
}

- (void)setScaled:(BOOL)scaled animated:(BOOL)animated
{
    _scaled = scaled;
    
    if(_scaled)
    {
        self.bounds = CGRectMake(0, 0, floorf(kTokenFieldCellScaledFactor*_size.width), floorf(kTokenFieldCellScaledFactor*_size.height));
        self.alpha = kTokenFieldCellScaledOpacity;
    }
    else
    {
        self.bounds = CGRectMake(0, 0, _size.width, _size.height);
        self.alpha = kTokenFieldCellOpacity;
    }
    
    if(animated)
    {
        if(scaled)
            self.transform = CGAffineTransformMakeScale(1.0f/kTokenFieldCellScaledFactor, 1.0f/kTokenFieldCellScaledFactor);
        else
            self.transform = CGAffineTransformMakeScale(kTokenFieldCellScaledFactor, kTokenFieldCellScaledFactor);
        
        [UIView animateWithDuration:kTokenFieldCellScaledAnimationDuration delay:0
                            options:(UIViewAnimationOptionCurveEaseInOut)
                         animations:^{
                             self.transform = CGAffineTransformMakeScale(1.0f, 1.0f);
                         }
                         completion:^(BOOL finished){
                             if(finished)
                                 self.transform = CGAffineTransformIdentity;
                             self.frame = CGRectMake(floorf(self.frame.origin.x), floorf(self.frame.origin.y), floorf(self.frame.size.width), floorf(self.frame.size.height));
                         }];
    }
}

#pragma mark -
#pragma mark UIControl

//- (void)sendAction:(SEL)action to:(id)target forEvent:(UIEvent *)event
//{
//    [super sendAction:action to:target forEvent:event];
//}

#pragma mark -
#pragma mark Drawing

- (void)drawRect:(CGRect)rect
{
    UIImage * backgroundImage;
    UIColor * textColor;
    UIFont * textFont;
    CGPoint textPoint;
    
    if(self.scaled)
    {
        NSString * selectedScaledPath = [[NSBundle bundleForClass:[self class]] pathForResource:@"FieldKit.bundle/token-atom-selected-scaled" ofType:@"png"];
        backgroundImage = [[UIImage imageWithContentsOfFile:selectedScaledPath] stretchableImageWithLeftCapWidth:kTokenFieldCellScaledStretchCap topCapHeight:kTokenFieldCellScaledStretchCap];
       
        textColor = [UIColor whiteColor];
        textFont = [UIFont fontWithName:_font.fontName size:floorf(_font.pointSize*kTokenFieldCellScaledFactor)];
        textPoint = CGPointMake(kTokenFieldCellScaledInsetLeft, kTokenFieldCellScaledInsetTop);
    }
    else
    {
        if(self.selected)
        {
            NSString * selectedPath = [[NSBundle bundleForClass:[self class]] pathForResource:@"FieldKit.bundle/token-atom-selected" ofType:@"png"];
            backgroundImage = [[UIImage imageWithContentsOfFile:selectedPath] stretchableImageWithLeftCapWidth:kTokenFieldCellStretchCap topCapHeight:kTokenFieldCellStretchCap];
            
            textColor = [UIColor whiteColor];
        }
        else
        {
            NSString * path = [[NSBundle bundleForClass:[self class]] pathForResource:@"FieldKit.bundle/token-atom" ofType:@"png"];
            backgroundImage = [[UIImage imageWithContentsOfFile:path] stretchableImageWithLeftCapWidth:kTokenFieldCellStretchCap topCapHeight:kTokenFieldCellStretchCap];
            
            textColor = [UIColor blackColor];
        }
        
        textFont = _font;
        textPoint = CGPointMake(kTokenFieldCellInsetLeft, kTokenFieldCellInsetTop);
    }
    
    // Draw background image
    [backgroundImage drawInRect:rect];
    
    // Draw text
    [textColor setFill];
    [_text drawAtPoint:textPoint withFont:textFont];
}

#pragma mark -
#pragma mark Helpers

@end
